package com.zh.project0512.controller.manage;

import com.zh.project0512.annotation.AdminCheckAuthorityAnnotation;
import com.zh.project0512.model.dto.OperationRoleJurisdictionDTO;
import com.zh.project0512.model.dto.AdminJurisdictionFBIDTO;
import com.zh.project0512.model.vo.AdminJurisdictionFBRIVo;
import com.zh.project0512.service.IAdminJurisdictionService;
import com.zh.project0512.utils.BizException;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import com.zh.project0512.utils.result.ResultList;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;


/**
 * 角色权限
 */
@Tag(name = "角色权限")
@RestController
@RequestMapping("/manage/userJurisdiction")
public class AdminJurisdictionController {
    @Resource
    IAdminJurisdictionService userJurisdictionService;

    /**
     * 操作角色权限
     * @param operationRoleJurisdictionDTO 操作用户权限DTO
     * @return 操作结果
     */
    @Operation(summary = "操作角色权限")
    @PostMapping("/operationRoleJurisdiction")
    @AdminCheckAuthorityAnnotation(jurisdictionId = "130")
    public Result<String> operationRoleJurisdiction(@Validated @RequestBody OperationRoleJurisdictionDTO operationRoleJurisdictionDTO){
        if (userJurisdictionService.operationRoleJurisdiction(operationRoleJurisdictionDTO)){
            return Result.success("操作角色权限","操作成功");
        }else {
            throw new BizException(HttpStatusEnum.CUSTOM_EXCEPTION,"操作失败");
        }
    }

    /**
     * 根据角色主键id查询角色权限信息
     * @param adminJurisdictionFBIDTO 角色权限 FBI DTO
     * @return 角色权限信息
     */
    @Operation(summary = "根据角色主键id查询角色权限信息")
    @PostMapping("/findByRoleId")
    public ResultList<AdminJurisdictionFBRIVo> findByRoleId(@Validated @RequestBody AdminJurisdictionFBIDTO adminJurisdictionFBIDTO){
        return ResultList.success(userJurisdictionService.findByRoleId(adminJurisdictionFBIDTO));
    }
}
