package com.zh.project0512.controller.manage;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zh.project0512.annotation.AdminCheckAuthorityAnnotation;
import com.zh.project0512.annotation.AdminTokenValid;
import com.zh.project0512.model.RewardRule;
import com.zh.project0512.model.RewardRuleTemplate;
import com.zh.project0512.service.IRewardRuleService;
import com.zh.project0512.service.IRewardRuleTemplateService;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-23
 */
@RestController
@RequestMapping("/manage/rewardRuleTemplate")
@Tag(name = "奖励规则模版")
public class RewardRuleTemplateController {
    @Autowired
    private IRewardRuleTemplateService rewardRuleTemplateService;

    @Autowired
    private IRewardRuleService rewardRuleService;

    @Data
    static class AddRRTParam {
        @NotNull(message = "title不能为空")
        @Schema(title = "模版名称")
        private String title;
        @NotNull(message = "rewardRule列表不能为空")
        @Valid
        @NotEmpty(message = "rewardRule列表至少1个元素")
        @Schema(title = "奖励规则列表")
        private RewardRule[] rewardRuleList;
    }

    @Operation(summary = "新增模版")
    @PostMapping("/add")
    @AdminTokenValid
    @Transactional
    @AdminCheckAuthorityAnnotation(jurisdictionId = "122")
    public Result add(@Validated @RequestBody AddRRTParam param) {
        List<RewardRule> rr = Arrays.asList(param.getRewardRuleList());
        rr.sort(Comparator.comparing(RewardRule::getLimitNum));
        RewardRuleTemplate rewardRuleTemplate = new RewardRuleTemplate();
        rewardRuleTemplate.setTitle(param.getTitle()).setCreatAt(LocalDateTime.now());
        rewardRuleTemplateService.save(rewardRuleTemplate);
        int templateId = rewardRuleTemplate.getId();
        rewardRuleService.addGroup(templateId, param.getRewardRuleList());
        return Result.success("添加成功");
    }

    @Data
    static class DelRRTParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "模版id")
        private Integer id;
    }

    @Operation(summary = "删除模版")
    @PostMapping("/del")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "124")
    public Result del(@Validated @RequestBody DelRRTParam param) {
        return MybatisPlusUtil.sqlResult(rewardRuleTemplateService.removeById(param.getId()), "删除");
    }

    @Data
    static class UpdRRTParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "模版id")
        private Integer id;
        @Schema(title = "模版名称")
        @NotNull(message = "name不能为空")
        private String title;
    }

    @Operation(summary = "更新模版")
    @PostMapping("/upd")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "123")
    public Result upd(@Validated @RequestBody UpdRRTParam param) {
        return MybatisPlusUtil.sqlResult(rewardRuleTemplateService.updateById(new RewardRuleTemplate().setId(param.getId()).setTitle(param.getTitle()).setUpdateAt(LocalDateTime.now())), "修改");
    }

    @Data
    static class DefRRTParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "模版id")
        private Integer id;
    }

    @Operation(summary = "设置默认模版")
    @PostMapping("/setDefault")
    @AdminTokenValid
    @Transactional
    @AdminCheckAuthorityAnnotation(jurisdictionId = "123")
    public Result setDefault(@Validated @RequestBody DefRRTParam param) {
        rewardRuleTemplateService.setDefault(param.getId());
        return Result.success("修改完成");
    }

    @Operation(summary = "模版列表")
    @PostMapping("/list")
    @AdminTokenValid
    public Result list(@RequestBody(required = false) @Parameter(hidden = true) JSONObject obj) {
        IPage ipage = rewardRuleTemplateService.idList(MybatisPlusUtil.SetPage(obj));
        List idList = ipage.getRecords();
        if(idList.size()>0){
            ipage.setRecords(rewardRuleTemplateService.rrList(idList));
        }
//        List list = rewardRuleTemplateService.rewardRuleTemplateList((MybatisPlusUtil.SetPage(obj)));
        return Result.success(ipage);
    }

    @Data
    static class DetTaskParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "模版id")
        private Integer id;
    }

    @Operation(summary = "详情")
    @PostMapping("/detail")
    @AdminTokenValid
    public Result detail(@Validated @RequestBody TaskController.DetTaskParam param) {
        int id =param.getId();
        RewardRuleTemplate rewardRuleTemplate = rewardRuleTemplateService.rewardRuleTemplateDetail(id);
        return rewardRuleTemplate != null ? Result.success(rewardRuleTemplate) : Result.fail(HttpStatusEnum.NOT_FOUND);
    }

}

