package com.zh.project0512.controller.manage;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zh.project0512.annotation.AdminCheckAuthorityAnnotation;
import com.zh.project0512.annotation.AdminTokenValid;
import com.zh.project0512.model.Brand;
import com.zh.project0512.model.TopicActivity;
import com.zh.project0512.service.ITopicActivityService;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import net.sf.jsqlparser.statement.select.Top;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.time.LocalDateTime;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-26
 */
@Tag(name = "动态")
@RestController
@RequestMapping("/manage/topicActivity")
public class TopicActivityController {
    @Autowired
    private ITopicActivityService topicActivityService;

    @Data
    static class AddTopicParam {
        @Schema(title = "标题")
        @NotNull(message = "title不能为空")
        @Size(max=64,min = 1,message = "标题长度在不超过64")
        private String title;
        @Schema(title = "副标题")
        private String subtitle;
        @Schema(title = "类型", description = "1视频；2图片；3文章")
        @NotNull(message = "type不能为空")
        private Integer type;
        @NotNull(message = "content不能为空")
        private String content;
        @Schema(title = "时长", description = "视频文件请上传时长")
        private Integer duration;
        @Schema(title = "封面")
        private String cover;
        @Schema(title = "是否首页显示", description = "0否；1是")
        private Integer showAtIndex;
    }

    @Operation(summary = "新增动态")
    @PostMapping("/add")
    @Transactional
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "104")
    public Result add(@Validated @RequestBody AddTopicParam param) {
        TopicActivity topicActivity = new TopicActivity()
            .setTitle(param.getTitle())
            .setSubtitle(param.getSubtitle().trim())
            .setType(param.getType())
            .setContent(param.getContent())
            .setDuration(param.getDuration())
            .setCover(param.getCover())
            .setSortWeight(1)
            .setCreatAt(LocalDateTime.now())
            .setUpdateAt(LocalDateTime.now());
        if (param.getShowAtIndex() != null && param.getShowAtIndex() == 1) {
            topicActivity.setShowAtIndex(1);
        }
        topicActivityService.updSortWeightPlus();
        topicActivityService.save(topicActivity);
        return Result.success("添加完成");
    }

    @Data
    static class DelTopicParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "动态id")
        private Integer id;
    }

    @Operation(summary = "删除动态")
    @PostMapping("/del")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "106")
    public Result del(@Validated @RequestBody DelTopicParam param) {
        return MybatisPlusUtil.sqlResult(topicActivityService.removeById(param.getId()), "删除");
    }

    @Data
    static class UpdTopicParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "动态id")
        private Integer id;
        @Schema(title = "标题")
        @Size(max=64,min = 1,message = "标题长度在不超过64")
        private String title;
        @Schema(title = "副标题")
        private String subtitle;
        @Schema(title = "内容")
        private String content;
        @Schema(title = "类型", description = "1视频；2图片；3文章")
        private Integer type;
        @Schema(title = "封面")
        private String cover;
        @Schema(title = "时长", description = "视频文件请上传时长")
        private Integer duration;
    }

    @Operation(summary = "更新动态内容")
    @PostMapping("/upd")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "105")
    public Result upd(@Validated @RequestBody UpdTopicParam param) {
        UpdateWrapper<TopicActivity> qw = new UpdateWrapper<>();
        qw.eq("id", param.getId())
            .set(param.getTitle() != null, "title", param.getTitle())
            .set(param.getSubtitle() != null, "subtitle", param.getSubtitle())
            .set(param.getType() != null, "type", param.getType())
            .set(param.getCover() != null, "cover", param.getCover())
            .set(param.getDuration() != null, "duration", param.getDuration())
            .set(param.getContent() != null, "content", param.getContent())
            .set("updateAt", LocalDateTime.now());
        return MybatisPlusUtil.sqlResult(topicActivityService.update(qw), "修改");
    }

    @Data
    static class ListTopicParam {
        private Integer pageNum;
        private Integer pageSize;
        @Schema(title = "标题")
        private String keyword;
        @Schema(title = "是否为首页展示", description = "1:是，2否")
        private Boolean showAtIndex;
    }

    @Operation(summary = "动态列表")
    @PostMapping("/list")
    @AdminTokenValid
    public Result list(@Validated @RequestBody ListTopicParam param) {
        QueryWrapper<TopicActivity> qw = new QueryWrapper<>();
        qw.like(param.getKeyword() != null, "title", param.getKeyword());
        qw.orderByDesc("showAtIndex");
        qw.orderByAsc("sortWeight");
        qw.orderByDesc("updateAt", "creatAt");
        if (param.getShowAtIndex() != null && param.getShowAtIndex() == true) {
            qw.eq("showAtIndex", 1);
        }
        return Result.success(topicActivityService.pageMaps(MybatisPlusUtil.SetNumPage(param.getPageNum(), param.getPageSize()), qw));
    }

    @Data
    static class DetTaskParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "动态id")
        private Integer id;
    }

    @Operation(summary = "详情")
    @PostMapping("/detail")
    @AdminTokenValid
    public Result detail(@Validated @RequestBody TaskController.DetTaskParam param) {
        int id = param.getId();
        TopicActivity topicActivity = topicActivityService.getById(id);
        return topicActivity != null ? Result.success(topicActivity) : Result.fail(HttpStatusEnum.NOT_FOUND);
    }

    @Data
    static class showAtIndexParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "动态id")
        private Integer id;
        @NotNull(message = "showAtIndex不能为空")
        @Min(value = 0, message = "取值是：0否；1是")
        @Max(value = 1, message = "取值是：0否；1是")
        @Schema(title = "是否首页显示", description = "0否；1是")
        private Integer showAtIndex;
    }

    @Operation(summary = "修改动态是否首页显示")
    @PostMapping("/showAtIndex")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "105")
    public Result showAtIndex(@Validated @RequestBody showAtIndexParam param) {
        UpdateWrapper<TopicActivity> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", param.getId()).set("showAtIndex", param.getShowAtIndex());
        return MybatisPlusUtil.sqlResult(topicActivityService.update(updateWrapper), "编辑");
    }

    @Data
    static class UpdTAParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "动态id")
        private Integer id;
        @NotNull(message = "排序权重不能为空")
        @Min(value = 1, message = "排序权重最小值为1")
        @Schema(title = "排序权重:数字越小排序靠前")
        private Integer sortWeight;
    }

    @Operation(summary = "修改排序")
    @PostMapping("/updSort")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "137")
    public Result updSort(@Validated @RequestBody UpdTAParam param) {
        TopicActivity topicActivity = topicActivityService.getById(param.getId());
        int origin = topicActivity.getSortWeight();
        int current = param.getSortWeight();
        if(origin==current){
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION,"权重无变化");
        }
        topicActivityService.updSort(origin,current,origin>current);
        return Result.success("修改完成");
    }

}
