package com.zh.project0512.controller.manage;


import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zh.project0512.annotation.AdminCheckAuthorityAnnotation;
import com.zh.project0512.annotation.AdminTokenValid;
import com.zh.project0512.model.VideoEffectSetting;
import com.zh.project0512.service.IVideoEffectSettingService;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-06-14
 */
@Tag(name = "视频传播系数管理")
@RestController
@RequestMapping("/manage/videoEffectSetting")
public class VideoEffectSettingController {
    @Autowired
    private IVideoEffectSettingService videoEffectSettingService;

    @Operation(summary = "列表")
    @PostMapping("/list")
    @AdminTokenValid
    public Result list() {
        return Result.success(videoEffectSettingService.list());
    }

    @Data
    static class UpdVESParam {
        @NotNull(message = "参数名不能为空")
        @Schema(title = "参数名")
        private String paramName;
        @NotNull(message = "传播系数不能为空")
        @Min(value = 1, message = "系数最小值为1")
        @Schema(title = "传播系数")
        private Integer effectTimes;
    }

    @Operation(summary = "设置系数", description = "整数")
    @PostMapping("/upd")
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "149")
    public Result upd(@Validated @RequestBody UpdVESParam param) {
        UpdateWrapper<VideoEffectSetting> up = new UpdateWrapper<>();
        up.eq("paramName", param.getParamName()).set("effectTimes", param.getEffectTimes());
        return MybatisPlusUtil.sqlResult(videoEffectSettingService.update(up), "修改");
    }

}
