package com.zh.project0512.serviceImpl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.zh.project0512.mapper.AdminRoleJurisdictionMapper;
import com.zh.project0512.mapper.AdminRoleMapper;
import com.zh.project0512.model.AdminRole;
import com.zh.project0512.model.AdminRoleJurisdiction;
import com.zh.project0512.model.dto.AdminRoleInsertDTO;
import com.zh.project0512.model.dto.AdminRoleListDTO;
import com.zh.project0512.model.dto.AdminRoleUpdateDTO;
import com.zh.project0512.model.vo.AdminRoleListVo;
import com.zh.project0512.service.IAdminRoleService;
import com.zh.project0512.utils.BizException;
import com.zh.project0512.utils.PropertyUtils;
import com.zh.project0512.utils.page.PageInfo;
import com.zh.project0512.utils.result.HttpStatusEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;

@Service
public class AdminRoleServiceImpl implements IAdminRoleService {
    @Resource
    AdminRoleMapper adminRoleMapper;
    @Resource
    AdminRoleJurisdictionMapper adminRoleJurisdictionMapper;

    @Override
    public PageInfo<AdminRoleListVo> list(AdminRoleListDTO adminRoleListDTO) {
        ArrayList<AdminRoleListVo> adminRoleListVoList = new ArrayList<>();
        QueryWrapper<AdminRole> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(adminRoleListDTO.getName()),"name", adminRoleListDTO.getName());

        Page<AdminRole> page = new Page<>(adminRoleListDTO.getPageNum(), adminRoleListDTO.getPageSize());
        IPage<AdminRole> userRoleIPage = adminRoleMapper.selectPage(page, queryWrapper);
        List<AdminRole> records = userRoleIPage.getRecords();
        if (records != null && records.size()>0){
            for (AdminRole record : records) {
                AdminRoleListVo adminRoleListVo = new AdminRoleListVo();
                PropertyUtils.copyProperties(record, adminRoleListVo);
                adminRoleListVoList.add(adminRoleListVo);
            }
        }
        return new PageInfo<>(userRoleIPage.getPages(), adminRoleListVoList, userRoleIPage.getTotal());
    }

    @Override
    public boolean insert(AdminRoleInsertDTO adminRoleInsertDTO) {
        AdminRole adminRole = new AdminRole();
        adminRole.setName(adminRoleInsertDTO.getName());
        adminRole.setCode(UUID.randomUUID().toString().replace("-",""));
        adminRole.setCreateAt(new Date());
        int insert = adminRoleMapper.insert(adminRole);
        return insert > 0;
    }

    @Override
    public boolean update(AdminRoleUpdateDTO adminRoleUpdateDTO) {
        AdminRole adminRole = adminRoleMapper.selectById(adminRoleUpdateDTO.getId());
        if (adminRole == null){
            throw new BizException(HttpStatusEnum.CUSTOM_EXCEPTION,"记录不存在");
        }

        adminRole.setName(adminRoleUpdateDTO.getName());
        adminRole.setModifyAt(new Date());
        int update = adminRoleMapper.updateById(adminRole);
        return update > 0;
    }

    @Override
    public List<Integer> findJurisdictionByRoleId(String roleId) {
        ArrayList<Integer> jurisdictionIds = new ArrayList<>();

        QueryWrapper<AdminRoleJurisdiction> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("roleId",roleId);
        List<AdminRoleJurisdiction> adminRoleJurisdictionList = adminRoleJurisdictionMapper.selectList(queryWrapper);
        for (AdminRoleJurisdiction adminRoleJurisdiction : adminRoleJurisdictionList) {
            jurisdictionIds.add(adminRoleJurisdiction.getJurisdictionId());
        }

        return jurisdictionIds;
    }
}
